@@ -1,4 +1,4 @@
-import sys
+import os
 
 def check(a, b):
     if a != b:
@@ -232,4 +232,7 @@ def test_url():
     'foo/bar/baz'
     """
 
+if 'TERM' in os.environ:
+    del os.environ['TERM']
+
 doctest.testmod(optionflags=doctest.NORMALIZE_WHITESPACE)
